@@ -1,4 +1,5 @@
 """A cache of javascipt files, optionally compressed."""
+from __future__ import unicode_literals
 
 from django.conf import settings
 from django.core.files.base import ContentFile
@@ -55,7 +56,7 @@ class JavascriptCache(object):
             force_save = not settings.DEBUG
         if force_save:
             if assets:
-                return [self._asset_cache.get_url(JavascriptAsset(map(AdaptiveAsset, assets), compile, rescope), force_save=True)]
+                return [self._asset_cache.get_url(JavascriptAsset(list(map(AdaptiveAsset, assets)), compile, rescope), force_save=True)]
             return []
         return [self._asset_cache.get_url(asset) for asset in assets]
         
