@@ -1,9 +1,10 @@
 """Template tags used for optimizing assets."""
-
-from urlparse import urlparse
+from __future__ import unicode_literals
 
 from django import template
 from django.utils.html import escape
+from django.utils import six
+from django.utils.six.moves.urllib.parse import urlparse
 
 from optimizations.assetcache import StaticAsset, default_asset_cache, AdaptiveAsset
 from optimizations.thumbnailcache import default_thumbnail_cache, PROPORTIONAL, ThumbnailError
@@ -26,7 +27,7 @@ def asset(src):
 @assignment_tag(register)
 def get_asset(src):
     return default_asset_cache.get_url(src)
-    
+
 
 @inclusion_tag(register, "assets/img.html")
 @assignment_tag(register, name="get_img")
@@ -55,7 +56,7 @@ def img(src, width=None, height=None, method=PROPORTIONAL, alt="", **attrs):
             "url": thumbnail.url,
             "width": thumbnail.width,
             "height": thumbnail.height,
-            
+
         })
     return params
 
@@ -81,7 +82,7 @@ def video_img(src, width, height, method=VIDEO_PROPORTIONAL, alt="", **attrs):
 
 def is_url(s):
     """Checks if the given string is a URL."""
-    if not isinstance(s, basestring):
+    if not isinstance(s, six.string_types):
         return False
     src_parts = urlparse(s)
     return src_parts.scheme or src_parts.netloc
@@ -90,7 +91,7 @@ def is_url(s):
 def resolve_script_src(src, _src):
     """Resolves the given src attribute of the script."""
     all_src = (src,) + _src
-    src_urls = filter(is_url, all_src)
+    src_urls = list(filter(is_url, all_src))
     if src_urls:
         if len(src_urls) == len(all_src):
             return all_src  # All are URLs, which is allowed.
@@ -98,8 +99,8 @@ def resolve_script_src(src, _src):
             raise ValueError("Mixed assets and absolute URLs are not allowed in script tags.")
     assets = StaticAsset.load("js", all_src)
     return default_javascript_cache.get_urls(assets)
-    
-    
+
+
 @inclusion_tag(register, "assets/script.html")
 def script(src="default", *_src, **attrs):
     """Renders one or more script tags."""
@@ -108,8 +109,8 @@ def script(src="default", *_src, **attrs):
         "urls": urls,
         "attrs": attrs,
     }
-    
-    
+
+
 @inclusion_tag(register, "assets/script_async.html")
 def script_async(src="default", *_src):
     """Renders an asyncronously-loading script tag."""
@@ -117,14 +118,14 @@ def script_async(src="default", *_src):
     return {
         "urls": urls,
     }
-    
-    
+
+
 @inclusion_tag(register, "assets/stylesheet.html")
 def stylesheet(href="default", *_href, **attrs):
     """Renders one or more stylesheet tags."""
     compile = attrs.pop("compile", True)
     all_href = (href,) + _href
-    href_urls = filter(is_url, all_href)
+    href_urls = list(filter(is_url, all_href))
     if href_urls:
         if len(href_urls) == len(all_href):
             urls = all_href  # All are URLs, which is allowed.
@@ -136,4 +137,4 @@ def stylesheet(href="default", *_href, **attrs):
     return {
         "urls": urls,
         "attrs": attrs,
-    }
\ No newline at end of file
+    }
